@@ -7,12 +7,12 @@ from fastapi_amis_admin.admin.site import AdminSite
 app = FastAPI()
 
 # 创建AdminSite实例
-site = AdminSite(settings=Settings(database_url_async='sqlite+aiosqlite:///amisadmin.db'))
+site = AdminSite(settings=Settings(database_url_async="sqlite+aiosqlite:///amisadmin.db"))
 
 # 挂载后台管理系统
 site.mount_app(app)
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import uvicorn
 
     uvicorn.run(app, debug=True)
