@@ -1,15 +1,16 @@
+import asyncio
 import datetime
 import re
 from functools import lru_cache
 from typing import (
     Any,
+    Awaitable,
     Callable,
     Dict,
     Generic,
     Iterable,
     Iterator,
     List,
-    NewType,
     Optional,
     Tuple,
     Type,
@@ -65,17 +66,13 @@ from fastapi_amis_admin.amis.constants import (
 )
 from fastapi_amis_admin.amis.types import (
     AmisAPI,
+    AmisNode,
     BaseAmisApiOut,
     BaseAmisModel,
     SchemaNode,
 )
 from fastapi_amis_admin.crud import RouterMixin, SQLModelCrud
-from fastapi_amis_admin.crud.base import (
-    SchemaCreateT,
-    SchemaFilterT,
-    SchemaModelT,
-    SchemaUpdateT,
-)
+from fastapi_amis_admin.crud.base import SchemaCreateT, SchemaFilterT, SchemaUpdateT
 from fastapi_amis_admin.crud.parser import (
     SQLModelFieldParser,
     SQLModelListField,
@@ -92,9 +89,10 @@ from fastapi_amis_admin.crud.utils import (
 from fastapi_amis_admin.utils.functools import cached_property
 from fastapi_amis_admin.utils.translation import i18n as _
 
-_BaseAdminT = TypeVar("_BaseAdminT", bound="BaseAdmin")
-_PageSchemaAdminT = TypeVar("_PageSchemaAdminT", bound="PageSchemaAdmin")
-_BaseModel = NewType("_BaseModel", BaseModel)
+BaseAdminT = TypeVar("BaseAdminT", bound="BaseAdmin")
+PageSchemaAdminT = TypeVar("PageSchemaAdminT", bound="PageSchemaAdmin")
+ActionT = Union[Action, Awaitable[Action]]
+ActionAdminT = Union["FormAdmin", "ModelAdmin"]
 
 
 class LinkModelForm:
@@ -332,6 +330,7 @@ class BaseModelAdmin(SQLModelCrud):
     enable_bulk_create: bool = False  # whether to enable batch creation
     search_fields: List[SQLModelListField] = []  # fuzzy search fields
     page_schema: Union[PageSchema, str] = PageSchema()
+    registered_admin_actions: Dict[str, "AdminAction"] = {}
 
     def __init__(self, app: "AdminApp"):
         assert self.model, "model is None"
@@ -341,7 +340,7 @@ class BaseModelAdmin(SQLModelCrud):
         self.amis_parser = self.app.site.amis_parser
         self.parser = SQLModelFieldParser(default_model=self.model)
         list_display_insfield = self.parser.filter_insfield(self.list_display, save_class=(Label,))
-        self.list_filter = self.list_filter or list_display_insfield or [self.model]
+        self.list_filter = self.list_filter and self.list_filter.copy() or list_display_insfield or [self.model]
         self.list_filter.extend(self.search_fields)
         super().__init__(self.model, self.engine)
         self.fields.extend(list_display_insfield)
@@ -391,6 +390,18 @@ class BaseModelAdmin(SQLModelCrud):
                 modelfield = self.parser.get_modelfield(field)
                 if modelfield:
                     columns.append(await self.get_list_column(request, modelfield))
+        # Append operation column
+        actions = await self.get_actions(request, flag="column")
+        if actions:
+            columns.append(
+                ColumnOperation(
+                    width=160,
+                    label=_("Operation"),
+                    breakpoint="*",
+                    buttons=actions,
+                )
+            )
+        # Append inline link model column
         for link_form in self.link_model_forms:
             form = await link_form.get_form_item(request)
             if form:
@@ -438,7 +449,7 @@ class BaseModelAdmin(SQLModelCrud):
                 "align": "right",
             },
         ]
-        headerToolbar.extend(await self.get_actions_on_header_toolbar(request))
+        headerToolbar.extend(await self.get_actions(request, flag="toolbar"))
         table = TableCRUD(
             api=await self.get_list_table_api(request),
             autoFillHeight=True,
@@ -449,8 +460,8 @@ class BaseModelAdmin(SQLModelCrud):
             syncLocation=False,
             keepItemSelectionOnPageChange=True,
             perPage=self.list_per_page,
-            itemActions=await self.get_actions_on_item(request),
-            bulkActions=await self.get_actions_on_bulk(request),
+            itemActions=await self.get_actions(request, flag="item"),
+            bulkActions=await self.get_actions(request, flag="bulk"),
             footerToolbar=[
                 "statistics",
                 "switch-per-page",
@@ -617,8 +628,6 @@ class BaseModelAdmin(SQLModelCrud):
         )
 
     async def get_create_action(self, request: Request, bulk: bool = False) -> Optional[Action]:
-        if not await self.has_create_permission(request, None):
-            return None
         if not bulk:
             return ActionType.Dialog(
                 icon="fa fa-plus pull-left",
@@ -642,9 +651,6 @@ class BaseModelAdmin(SQLModelCrud):
         )
 
     async def get_update_action(self, request: Request, bulk: bool = False) -> Optional[Action]:
-        if not await self.has_update_permission(request, None, None):
-            return None
-        # Turn on batch editing
         if not bulk:
             return ActionType.Dialog(
                 icon="fa fa-pencil",
@@ -655,7 +661,6 @@ class BaseModelAdmin(SQLModelCrud):
                     body=await self.get_update_form(request, bulk=bulk),
                 ),
             )
-
         elif self.bulk_update_fields:
             return ActionType.Dialog(
                 label=_("Bulk Update"),
@@ -665,50 +670,24 @@ class BaseModelAdmin(SQLModelCrud):
                     body=await self.get_update_form(request, bulk=True),
                 ),
             )
-
         else:
             return None
 
-    async def get_delete_action(self, request: Request, bulk: bool = False) -> Optional[Action]:
-        if not await self.has_delete_permission(request, None):
-            return None
-        return (
-            ActionType.Ajax(
-                label=_("Bulk Delete"),
-                confirmText=_("Are you sure you want to delete the selected rows?"),
-                api=f"delete:{self.router_path}/item/" + "${ids|raw}",
-            )
-            if bulk
-            else ActionType.Ajax(
-                icon="fa fa-times text-danger",
-                tooltip=_("Delete"),
-                confirmText=_("Are you sure you want to delete row ${%s}?") % self.pk_name,
-                api=f"delete:{self.router_path}/item/${self.pk_name}",
-            )
-        )
+    async def get_action(self, request: Request, name: str) -> Action:
+        admin_action = self.registered_admin_actions.get(name)
+        return await admin_action.get_action(request)
 
-    async def get_actions_on_header_toolbar(self, request: Request) -> List[Action]:
-        actions = [
-            await self.get_create_action(request, bulk=False),
-        ]
-        if self.enable_bulk_create:
-            actions.append(await self.get_create_action(request, bulk=True))
+    async def get_actions(self, request: Request, flag: str) -> List[Action]:
+        actions = []
+        for admin_action in self.registered_admin_actions.values():
+            if flag not in admin_action.flags:
+                continue
+            if await self.has_action_permission(request, name=admin_action.name):
+                actions.append(await admin_action.get_action(request, name=admin_action.name))
         return list(filter(None, actions))
 
-    async def get_actions_on_item(self, request: Request) -> List[Action]:
-        actions = [
-            await self.get_read_action(request),
-            await self.get_update_action(request, bulk=False),
-            await self.get_delete_action(request, bulk=False),
-        ]
-        return list(filter(None, actions))
-
-    async def get_actions_on_bulk(self, request: Request) -> List[Action]:
-        bulkActions = [
-            await self.get_update_action(request, bulk=True),
-            await self.get_delete_action(request, bulk=True),
-        ]
-        return list(filter(None, bulkActions))
+    async def has_action_permission(self, request: Request, name: str) -> bool:
+        return True
 
     async def _conv_modelfields_to_formitems(
         self,
@@ -748,7 +727,6 @@ class BaseAdmin:
 
 
 class PageSchemaAdmin(BaseAdmin):
-    group_schema: Union[PageSchema, str] = None  # todo Deprecated
     page_schema: Union[PageSchema, str] = PageSchema()
 
     def __init__(self, app: "AdminApp"):
@@ -756,10 +734,9 @@ class PageSchemaAdmin(BaseAdmin):
         self.page_schema = self.get_page_schema()
         if self.page_schema and self.page_schema.url:
             self.page_schema.url = self.page_schema.url.replace(self.site.settings.site_url, "")
-        self.group_schema = self.get_group_schema()
 
-    async def has_page_permission(self, request: Request) -> bool:
-        return self.app is self or await self.app.has_page_permission(request)
+    async def has_page_permission(self, request: Request, obj: "PageSchemaAdmin" = None, action: str = None) -> bool:
+        return self.app is self or await self.app.has_page_permission(request, obj=obj or self, action=action)
 
     def get_page_schema(self) -> Optional[PageSchema]:
         if self.page_schema:
@@ -772,17 +749,6 @@ class PageSchemaAdmin(BaseAdmin):
                 raise TypeError()
         return self.page_schema
 
-    def get_group_schema(self) -> Optional[PageSchema]:
-        if self.group_schema:
-            if isinstance(self.group_schema, str):
-                self.group_schema = PageSchema(label=self.group_schema)
-            elif isinstance(self.group_schema, PageSchema):
-                self.group_schema = self.group_schema.copy(deep=True)
-                self.group_schema.label = self.group_schema.label or "default"
-            else:
-                raise TypeError()
-        return self.group_schema
-
 
 class LinkAdmin(PageSchemaAdmin):
     link: str = ""
@@ -842,7 +808,7 @@ class PageAdmin(PageSchemaAdmin, RouterAdmin):
         PageSchemaAdmin.__init__(self, app)
 
     async def page_permission_depend(self, request: Request) -> bool:
-        return await self.has_page_permission(request) or self.error_no_page_permission(request)
+        return await self.has_page_permission(request, action="page") or self.error_no_page_permission(request)
 
     def error_no_page_permission(self, request: Request):
         raise HTTPException(
@@ -946,7 +912,6 @@ class BaseFormAdmin(PageAdmin, Generic[SchemaUpdateT]):
     form: Form = None
     form_init: bool = None
     form_path: str = ""
-    route_init: Callable = None
     route_submit: Callable = None
     router_prefix: str = "/form"
 
@@ -998,6 +963,16 @@ class BaseFormAdmin(PageAdmin, Generic[SchemaUpdateT]):
             )
         return self
 
+    async def get_init_data(self, request: Request, **kwargs) -> BaseApiOut[Any]:
+        return BaseApiOut()
+
+    @property
+    def route_init(self):
+        async def route(request: Request):
+            return await self.get_init_data(request)
+
+        return route
+
 
 class FormAdmin(BaseFormAdmin):
     """Form management"""
@@ -1014,22 +989,13 @@ class FormAdmin(BaseFormAdmin):
     async def handle(self, request: Request, data: SchemaUpdateT, **kwargs) -> BaseApiOut[Any]:
         raise NotImplementedError
 
-    async def get_init_data(self, request: Request, **kwargs) -> BaseApiOut[Any]:
-        raise NotImplementedError
-
-    @property
-    def route_init(self):
-        async def route(request: Request):
-            return await self.get_init_data(request)
-
-        return route
-
 
 class ModelAdmin(BaseModelAdmin, PageAdmin):
     """Model management"""
 
     page_path: str = ""
     bind_model: bool = True
+    admin_action_maker: List[Callable[["ModelAdmin"], "AdminAction"]] = []  # Actions
 
     def __init__(self, app: "AdminApp"):
         BaseModelAdmin.__init__(self, app)
@@ -1041,9 +1007,80 @@ class ModelAdmin(BaseModelAdmin, PageAdmin):
             return f"/{self.__class__.__name__}"
         return f"/{self.model.__name__}"
 
+    @cached_property
+    def registered_admin_actions(self) -> Dict[str, "AdminAction"]:
+        admin_actions = {
+            "create": AdminAction(
+                admin=self,
+                name="create",
+                label=_("Create"),
+                flags=["toolbar"],
+                getter=lambda request: self.get_create_action(request, bulk=False),
+            ),
+            "update": AdminAction(
+                admin=self,
+                name="update",
+                tooltip=_("Update"),
+                flags=["item"],
+                getter=lambda request: self.get_update_action(request, bulk=False),
+            ),
+            "delete": AdminAction(
+                admin=self,
+                name="delete",
+                action=ActionType.Ajax(
+                    icon="fa fa-times text-danger",
+                    tooltip=_("Delete"),
+                    confirmText=_("Are you sure you want to delete row ${%s}?") % self.pk_name,
+                    api=f"delete:{self.router_path}/item/${self.pk_name}",
+                ),
+                flags=["item"],
+            ),
+            "bulk_delete": AdminAction(
+                admin=self,
+                name="bulk_delete",
+                action=ActionType.Ajax(
+                    label=_("Bulk Delete"),
+                    confirmText=_("Are you sure you want to delete the selected rows?"),
+                    api=f"delete:{self.router_path}/item/" + "${ids|raw}",
+                ),
+                flags=["bulk"],
+            ),
+        }
+        if self.enable_bulk_create:
+            admin_actions["bulk_create"] = AdminAction(
+                admin=self,
+                name="bulk_create",
+                label=_("Bulk Create"),
+                flags=["toolbar"],
+                getter=lambda request: self.get_create_action(request, bulk=True),
+            )
+        if self.schema_read:
+            admin_actions["read"] = AdminAction(
+                admin=self,
+                name="read",
+                label=_("View"),
+                flags=["item"],
+                getter=lambda request: self.get_read_action(request),
+            )
+        if self.bulk_update_fields:
+            admin_actions["bulk_update"] = AdminAction(
+                admin=self,
+                name="bulk_update",
+                label=_("Bulk Update"),
+                flags=["bulk"],
+                getter=lambda request: self.get_update_action(request, bulk=True),
+            )
+        for maker in self.admin_action_maker:
+            admin_action = maker(self)
+            admin_actions[admin_action.name] = admin_action
+        return admin_actions
+
     def register_router(self):
         for form in self.link_model_forms:
             form.register_router()
+        for admin_action in self.registered_admin_actions.values():
+            if isinstance(admin_action, RouterAdmin):
+                admin_action.register_router()
         self.register_crud()
         super(ModelAdmin, self).register_router()
         return self
@@ -1060,13 +1097,13 @@ class ModelAdmin(BaseModelAdmin, PageAdmin):
         filters: SchemaFilterT = None,
         **kwargs,
     ) -> bool:
-        return await self.has_page_permission(request)
+        return await self.has_page_permission(request, action=CrudEnum.list)
 
     async def has_create_permission(self, request: Request, data: SchemaCreateT, **kwargs) -> bool:  # type self.schema_create
-        return await self.has_page_permission(request)
+        return await self.has_page_permission(request, action=CrudEnum.create)
 
     async def has_read_permission(self, request: Request, item_id: List[str], **kwargs) -> bool:
-        return await self.has_page_permission(request)
+        return await self.has_page_permission(request, action=CrudEnum.read)
 
     async def has_update_permission(
         self,
@@ -1075,84 +1112,115 @@ class ModelAdmin(BaseModelAdmin, PageAdmin):
         data: SchemaUpdateT,
         **kwargs,
     ) -> bool:
-        return await self.has_page_permission(request)
+        return await self.has_page_permission(request, action=CrudEnum.update)
 
     async def has_delete_permission(self, request: Request, item_id: List[str], **kwargs) -> bool:
-        return await self.has_page_permission(request)
+        return await self.has_page_permission(request, action=CrudEnum.delete)
+
+    async def has_action_permission(self, request: Request, name: str) -> bool:
+        if not await self.has_page_permission(request, action=name):
+            return False
+        elif name in {"delete", "bulk_delete"}:
+            return await self.has_delete_permission(request, None)  # type: ignore
+        elif name in {"update", "bulk_update"}:
+            return await self.has_update_permission(request, None, None)  # type: ignore
+        elif name in {"create", "bulk_create"}:
+            return await self.has_create_permission(request, None)  # type: ignore
+        elif name in {"read"}:
+            return await self.has_read_permission(request, None)  # type: ignore
+        else:
+            return True
 
 
-class BaseFormAction:
-    admin: "FormAdmin" = None
+class AdminAction:
+    admin: ActionAdminT
     action: Action = None
 
-    def __init__(self, admin: "FormAdmin"):
+    def __init__(
+        self,
+        admin: ActionAdminT,
+        *,
+        name: str = None,
+        label: str = None,
+        action: Action = None,
+        flags: Union[str, List[str]] = None,
+        getter: Callable[[Request], ActionT] = None,
+        **kwargs,
+    ):
         self.admin = admin
         assert self.admin, "admin is None"
+        self.action = action or self.action or Action()
+        self.action = self.action.update_from_dict(kwargs)
+        self.name = name or self.action.id or self.action.name
+        assert self.name, "name is None"
+        self.label = label or self.action.label or self.action.tooltip
+        assert self.label, "label is None"
+        self.flags = flags or ["item"]
+        if isinstance(self.flags, str):
+            self.flags = [self.flags]
+        self.getter = getter
 
-    def register_router(self):
-        raise NotImplementedError
+    async def get_action(self, request: Request, **kwargs) -> Action:
+        if not self.getter:
+            return self.action
+        action = self.getter(request)
+        if asyncio.iscoroutine(action):
+            action = await action
+        return action
 
 
-class BaseModelAction:
-    admin: "ModelAdmin" = None
-    action: Action = None
+class FormAction(FormAdmin, AdminAction):
 
-    def __init__(self, admin: "ModelAdmin"):
-        self.admin = admin
-        assert self.admin, "admin is None"
+    action: Union[ActionType.Dialog, ActionType.Drawer]
 
-    async def fetch_item_scalars(self, item_id: List[str]) -> List[SchemaModelT]:
-        # noinspection PyProtectedMember
-        return await self.admin.db.async_run_sync(self.admin._fetch_item_scalars, item_id)
-
-    def register_router(self):
-        raise NotImplementedError
-
-
-class FormAction(FormAdmin, BaseFormAction):
-    schema: Type[BaseModel] = None
-    action: ActionType.Dialog = None
-
-    def __init__(self, admin: "FormAdmin"):
-        BaseFormAction.__init__(self, admin)
+    def __init__(
+        self,
+        admin: ActionAdminT,
+        *,
+        action: Action = None,
+        flags: List[str] = None,
+        getter: Callable[[ActionAdminT, Request], ActionT] = None,
+        **kwargs,
+    ):
+        AdminAction.__init__(self, admin, action=action, flags=flags, getter=getter, **kwargs)
         self.router = self.admin.router
         FormAdmin.__init__(self, self.admin.app)
 
     async def get_action(self, request: Request, **kwargs) -> Action:
-        action = self.action or ActionType.Dialog(label=_("Custom form actions"), dialog=Dialog())
-        action.dialog.title = action.dialog.title or action.label  # only override if not set
-        action.dialog.size = action.dialog.size or SizeEnum.xl
-        action.dialog.body = action.dialog.body or ""  # keep it empty for non model related custom form
+        action = self.action and self.action.copy() or ActionType.Dialog(label=_("Custom form actions"), dialog=Dialog())
+        node: AmisNode = getattr(action, action.actionType, None)
+        if node:
+            node.title = node.title or action.label  # only override if not set
+            node.size = node.size or SizeEnum.xl
+            node.body = node.body or ""  # keep it empty for non model related custom form
         return action
 
-    async def handle(self, request: Request, data: BaseModel, **kwargs) -> BaseApiOut[Any]:
+    async def handle(self, request: Request, data: SchemaUpdateT, **kwargs) -> BaseApiOut[Any]:
         return BaseApiOut(data=data)
 
 
-class ModelAction(BaseFormAdmin, BaseModelAction):
-    action: ActionType.Dialog = None
-
-    def __init__(self, admin: "ModelAdmin"):
-        BaseModelAction.__init__(self, admin)
-        self.router = self.admin.router
-        BaseFormAdmin.__init__(self, self.admin.app)
+class ModelAction(FormAction):
+    admin: ModelAdmin
 
     async def get_action(self, request: Request, **kwargs) -> Action:
-        action = self.action or ActionType.Dialog(label=_("Custom form actions"), dialog=Dialog())
-        action.dialog.title = action.dialog.title or action.label  # only override if not set
-        action.dialog.body = Service(
-            schemaApi=AmisAPI(
-                method="post",
-                url=self.router_path + self.page_path + "?item_id=${IF(ids, ids, id)}",
-                responseData={
-                    "&": "${body}",
-                    "api.url": "${body.api.url}?item_id=${api.query.item_id}",
-                    "submitText": "",
-                },
+        action = await super().get_action(request, **kwargs)
+        node: AmisNode = getattr(action, action.actionType, None)
+        if node:
+            node.body = Service(
+                schemaApi=AmisAPI(
+                    method="post",
+                    url=self.router_path + self.page_path + "?item_id=${IF(ids, ids, id)}",
+                    responseData={
+                        "&": "${body}",
+                        "api.url": "${body.api.url}?item_id=${api.query.item_id}",
+                        "initApi.url": "${body.initApi.url}?item_id=${api.query.item_id}" if self.form_init else None,
+                        "submitText": "",
+                    },
+                )
             )
-        )
         return action
 
+    # noinspection PyMethodOverriding
     async def handle(self, request: Request, item_id: List[str], data: Optional[SchemaUpdateT], **kwargs) -> BaseApiOut[Any]:
         return BaseApiOut(data=data)
 
@@ -1178,30 +1246,10 @@ class ModelAction(BaseFormAdmin, BaseModelAction):
 class AdminGroup(PageSchemaAdmin):
     def __init__(self, app: "AdminApp") -> None:
         super().__init__(app)
-        self._children: List[_PageSchemaAdminT] = []
+        self._children: List[PageSchemaAdminT] = []
 
-    @cached_property
-    def unique_id(self) -> str:
-        unique_str = super().unique_id
-        if self._children:
-            unique_str += self._children[0].unique_id
-        return md5_hex(unique_str)[:16]
-
-    def append_child(self, child: _PageSchemaAdminT, group_schema: PageSchema = None) -> None:
-        if not child.page_schema:
-            return
-        group_label = group_schema and group_schema.label
-        if not group_label:
-            self._children.append(child)
-        else:
-            for item in self._children:
-                if isinstance(item, AdminGroup) and item.page_schema and item.page_schema.label == group_label:
-                    item.append_child(child, group_schema=None)
-                    return
-            group = AdminGroup(self.app)
-            group.page_schema = group_schema.copy()
-            group.append_child(child, group_schema=None)
-            self._children.append(group)
+    def append_child(self, child: PageSchemaAdminT) -> None:
+        self._children.append(child)
 
     def remove_child(self, unique_id: str) -> None:
         self._children = [admin for admin in self._children if admin.unique_id != unique_id]
@@ -1212,23 +1260,23 @@ class AdminGroup(PageSchemaAdmin):
     async def get_page_schema_children(self, request: Request) -> List[PageSchema]:
         page_schema_list = []
         for child in self._children:
-            if not child.page_schema or not await child.has_page_permission(request):
+            if not child.page_schema:
                 continue
             if (isinstance(child, AdminGroup) and not isinstance(child, AdminApp)) or (
                 isinstance(child, AdminApp) and child.page_schema.tabsMode is None
             ):
                 sub_children = await child.get_page_schema_children(request)
-                if sub_children:
+                if sub_children:  # If there are sub-nodes, show them even if the parent node has no permission.
                     page_schema = child.page_schema.copy(deep=True)
                     page_schema.children = sub_children
                     page_schema_list.append(page_schema)
-            else:
+            elif await child.has_page_permission(request, action="page"):
                 page_schema_list.append(child.page_schema)
         if page_schema_list:
             page_schema_list.sort(key=lambda p: p.sort or 0, reverse=True)
         return page_schema_list
 
-    def get_page_schema_child(self, unique_id: str) -> Union[Tuple[_PageSchemaAdminT, "AdminGroup"], Tuple[None, None]]:
+    def get_page_schema_child(self, unique_id: str) -> Union[Tuple[PageSchemaAdminT, "AdminGroup"], Tuple[None, None]]:
         for child in self._children:
             if child.unique_id == unique_id:
                 return child, self
@@ -1238,7 +1286,7 @@ class AdminGroup(PageSchemaAdmin):
                     return child, parent
         return None, None
 
-    def __iter__(self) -> Iterator[_PageSchemaAdminT]:
+    def __iter__(self) -> Iterator[PageSchemaAdminT]:
         return self._children.__iter__()
 
 
@@ -1254,14 +1302,14 @@ class AdminApp(PageAdmin, AdminGroup):
         AdminGroup.__init__(self, app)
         self.engine = self.engine or self.app.engine
         self.db = get_engine_db(self.engine)
-        self._registered: Dict[Type[_BaseAdminT], Optional[_BaseAdminT]] = {}
+        self._registered: Dict[Type[BaseAdminT], Optional[BaseAdminT]] = {}
         self.__register_lock = False
 
     @property
     def router_prefix(self):
         return f"/{self.__class__.__name__}"
 
-    def get_admin_or_create(self, admin_cls: Type[_BaseAdminT], register: bool = True) -> Optional[_BaseAdminT]:
+    def get_admin_or_create(self, admin_cls: Type[BaseAdminT], register: bool = True) -> Optional[BaseAdminT]:
         if admin_cls not in self._registered and (not register or self.__register_lock):
             return None
         admin = self._registered.get(admin_cls)
@@ -1270,7 +1318,7 @@ class AdminApp(PageAdmin, AdminGroup):
         admin = admin_cls(self)
         self._registered[admin_cls] = admin
         if isinstance(admin, PageSchemaAdmin):
-            self.append_child(admin, group_schema=admin.group_schema)
+            self.append_child(admin)
         return admin
 
     def _create_admin_instance_all(self) -> None:
@@ -1306,7 +1354,7 @@ class AdminApp(PageAdmin, AdminGroup):
                     return admin
         return None
 
-    def register_admin(self, *admin_cls: Type[_BaseAdminT]) -> Type[_BaseAdminT]:
+    def register_admin(self, *admin_cls: Type[BaseAdminT]) -> Type[BaseAdminT]:
         [self._registered.update({cls: None}) for cls in admin_cls if cls]
         return admin_cls[0]
 
@@ -1388,15 +1436,15 @@ class BaseAdminSite(AdminApp):
 
     @cached_property
     def router_path(self) -> str:
-        return self.settings.site_url + self.settings.root_path + self.router.prefix
+        return self.settings.site_url + self.settings.site_path + self.router.prefix
 
     def mount_app(self, fastapi: FastAPI, name: str = "admin") -> None:
-        """mount app to fastapi, the path is: site.settings.root_path.
+        """mount app to fastapi, the path is: site.settings.site_path.
         once mount, the site will create all registered admin instance and register router.
         """
         self.application = fastapi
         self.register_router()
-        fastapi.mount(self.settings.root_path, self.fastapi, name=name)
+        fastapi.mount(self.settings.site_path, self.fastapi, name=name)
         fastapi.add_middleware(BaseHTTPMiddleware, dispatch=self.db.asgi_dispatch)
         """Add SQLAlchemy Session middleware to the main application, and the session object will be bound to each request.
         Note:
