@@ -6,10 +6,10 @@ def test_settings_valid_url():
     assert settings.site_url == ""
     settings = Settings(amis_cdn="https://unpkg.com/")
     assert settings.amis_cdn == "https://unpkg.com"
-    settings = Settings(root_path="/admin/")
-    assert settings.root_path == "/admin"
-    settings = Settings(root_path="/admin")
-    assert settings.root_path == "/admin"
+    settings = Settings(site_path="/admin/")
+    assert settings.site_path == "/admin"
+    settings = Settings(site_path="/admin")
+    assert settings.site_path == "/admin"
 
 
 def test_settings_valid_database_url():
