@@ -2,10 +2,10 @@ from fastapi_amis_admin.amis.components import Page, PageSchema
 
 
 def test_Page():
-    page = Page(title="标题", body="Hello World!")
-    page_json = r'{"type":"page","title":"\u6807\u9898","body":"Hello World!"}'
-    assert page.amis_json() == page_json
-    assert page.amis_dict() == {"type": "page", "title": "标题", "body": "Hello World!"}
+    page = Page(title="Title", body="HelloWorld!")
+    page_json = r'{"type":"page","title":"Title","body":"HelloWorld!"}'
+    assert page.amis_json().replace(" ", "") == page_json
+    assert page.amis_dict() == {"type": "page", "title": "Title", "body": "HelloWorld!"}
     assert page.amis_html().find(page_json)
 
 
